@@ -2,8 +2,8 @@
 # coding: utf-8
 
 import argparse
+from glob import glob
 import json
-import logging
 import multiprocessing as mp
 import os
 import platform
@@ -20,35 +20,37 @@ import tarfile
 import time
 import traceback
 from pathlib import Path
+from types import SimpleNamespace
 
 from rich.console import Console
+from loguru import logger
 from selenium import webdriver
 from selenium.common.exceptions import WebDriverException
 from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.firefox.options import Options
 from webdriver_manager.firefox import GeckoDriverManager
 
+from .config import Config
+
+console = Console()
+logger.remove()
+
+all_links = []
+
 
 class KeyboardInterruptHandler:
     def __init__(self):
         pass
+
     def keyboardInterruptHandler(*args):
         sys.tracebacklimit = 0
         print('', end='\r')
         time.sleep(0.5)
         console.print('[#f1fa8c]Quitting...')
-        logging.info('Interrupted by the user.')
+        logger.info('Interrupted by the user.')
         sys.exit(0)
 
 
-class Config:
-    PROJECT_PATH = f'{Path(__file__).parent}'
-    UBLOCK = f'{PROJECT_PATH}/ublock_latest.xpi'
-    DATA = f'{PROJECT_PATH}/data/servers_data.json'
-    LOG_FILE = f'{PROJECT_PATH}/pymirror.log'
-    WIN_GECKO = None
-
-
 class CustomHelpFormatter(argparse.HelpFormatter):
     def __init__(self, prog):
         super().__init__(prog, max_help_position=40, width=80)
@@ -65,66 +67,47 @@ def fmt(prog):
     return CustomHelpFormatter(prog)
 
 
-console = Console()
-
 class PyMirror:
-    def __init__(self, args):
-        self.args = args
+    def __init__(self):
         pass
 
-    def custom_logger(path: str):
-        myformat = '%(asctime)s - %(name)-12s - %(levelname)-8s - %(message)s'
-        # noinspection PyArgumentList
-        logging.basicConfig(level=logging.DEBUG,
-                            format=myformat,
-                            datefmt='%Y-%d-%m %H:%M:%S',
-                            handlers=[logging.FileHandler(path)])
-
-
     def custom_error_traceback(exception: Exception,
                                error_msg: str,
-                               log: bool=False,
-                               verbose: bool=False):
+                               log: bool = False):
         if log:
-            logging.error(error_msg)
-        tb = traceback.format_exception(None, exception, exception.__traceback__)
-        if verbose:
-            console.print(tb)
+            logger.error(error_msg)
+        tb = traceback.format_exception(None, exception,
+                                        exception.__traceback__)
         if log:
-            logging.error(f'{"-" * 10} Start of error traceback {"-" * 10}')
+            logger.error(f'{"-" * 10} Start of error traceback {"-" * 10}')
             for ln in tb:
-                logging.error(ln.replace('\n', ''))
-            logging.error(f'{"-" * 10} End of error traceback {"-" * 10}')
+                logger.error(ln.replace('\n', ''))
+            logger.error(f'{"-" * 10} End of error traceback {"-" * 10}')
         return tb
 
-
     def tgz(input_folder: str):
         out_file = f'{Path(input_folder).parent}/{Path(input_folder).name}.tar.gz'
         with tarfile.open(out_file, 'w:gz') as t:
             t.add(input_folder, arcname=Path(input_folder).name)
         return out_file
 
-
     def clean_filename(file: str):
         rfile = ''.join([
-            Path(x).stem.replace(x, "_") if x in string.punctuation + ' ' else x
-            for x in Path(Path(file).stem).stem
+            Path(x).stem.replace(x, "_") if x in string.punctuation +
+            ' ' else x for x in Path(Path(file).stem).stem
         ])
         rfile = f'{rfile}{"".join(Path(file).suffixes)}'
         rfile = f'{Path(file).parent}/{rfile}'
         os.rename(file, rfile)
         return rfile
 
-
     def initializer():
         signal.signal(signal.SIGINT, signal.SIG_IGN)
 
-
     def SeleniumExceptionInfo(exception: Exception):
-        exc_name = sys.exc_info()[0].__name__
-        logging.error(f'Selenium encountered an error: {exc_name}')
-        PyMirror.custom_error_traceback(exception, '!!! Selenium ERROR')
-
+        tb = traceback.format_exception(None, exception,
+                                        exception.__traceback__)
+        logger.error(f'Selenium encountered an error: {tb}')
 
     def return_ips(data: dict):
         ips = []
@@ -137,7 +120,6 @@ class PyMirror:
             ips.append((ip, k))
         return ips
 
-
     def ping(ip: str):
         if platform.system() == 'Windows':
             response = os.system(f'ping -n 1 {ip[0]} > /dev/null 2>&1')
@@ -145,16 +127,17 @@ class PyMirror:
             response = os.system(f'ping -c 1 {ip[0]} > /dev/null 2>&1')
         if response in [0, 256, 512]:
             console.print(
-                f'[[#50fa7b]OK[/#50fa7b]] [#8be9fd]{ip[1]}[/#8be9fd] is online!')
-            logging.info(f'{ip[1]} is online!')
+                f'[[#50fa7b] OK [/#50fa7b]] [#8be9fd]{ip[1]}[/#8be9fd] is online!'
+            )
+            logger.info(f'{ip[1]} is online!')
             return True
         else:
             console.print(
-                f'[[#ff5555]ERROR![/#ff5555]] [#8be9fd]{ip[1]}[/#8be9fd] is down!')
-            logging.warning(f'{ip[1]} is offline!')
+                f'[[#ff5555] ERROR! [/#ff5555]] [#8be9fd]{ip[1]}[/#8be9fd] is down!'
+            )
+            logger.warning(f'{ip[1]} is offline!')
             return False
 
-
     def curl(data: dict, server: str, file: str):
         srv = data[server]['server']
         keys = data[server]['keys']
@@ -182,9 +165,7 @@ class PyMirror:
 
         return link
 
-
     def api_uploads(args, data: dict, responses: list, rfile: str):
-        links_raw = []
         times = []
         for n, ((k, _), res) in enumerate(zip(data.items(), responses)):
             if args.number:
@@ -198,32 +179,22 @@ class PyMirror:
                 if len(times) > 2:
                     signal.alarm(int(statistics.mean(times)) + 5)
                 link = PyMirror.curl(data, k, rfile)
-                if 'Bad Gateway' in link:
+                if 'bad gateway' in link.lower() or 'error' in link.lower():
                     raise Exception
-                console.print('[[#50fa7b]OK[/#50fa7b]]', link)
-                logging.info(f'[OK] {link}')
-                links_raw.append(link)
+                console.print('[[#50fa7b] OK [/#50fa7b]]', link)
+                all_links.append(link)
+                logger.info(f'[ OK ] {link}')
                 times.append(time.time() - start)
             except ZeroDivisionError:
-                if args.verbose is True:
-                    console.print(
-                        f'[[#ff5555]ERROR![/#ff5555]] {k} Timed out! Skipping...')
                 if args.log:
-                    logging.error(f'{k} Timed out!')
+                    logger.error(f'{k} Timed out!')
             except Exception as e:
-                if args.verbose is True:
-                    error_class = sys.exc_info()[0].__name__
-                    console.print(
-                        f'[[#ff5555]ERROR![/#ff5555]] Error in {k}:'
-                        f' {error_class}. Skipping...'
-                    )
-                    PyMirror.custom_error_traceback(e,
-                                       f'[ERROR!] Error in {k}...',
-                                       log=args.log)
-            signal.alarm(0)
-
-        return links_raw
-
+                error_class = sys.exc_info()[0].__name__
+                PyMirror.custom_error_traceback(e,
+                                                f'[ ERROR! ] Error in {k}...',
+                                                log=args.log)
+            finally:
+                signal.alarm(0)
 
     def match_links(links_raw: list):
         regex = re.compile(
@@ -241,29 +212,47 @@ class PyMirror:
                 links.append(link)
         return links
 
-
     def download_ublock():
         latest = 'https://addons.mozilla.org/firefox/downloads/file/3806442'
-        os.popen(
-            f'curl -sLo {Config.PROJECT_PATH}/ublock_latest.xpi {latest}').read()
-        file_path = f'{Config.PROJECT_PATH}/config.py'
-        with open(file_path, 'r+') as f:
-            lines = f.readlines().copy()
-            with open(file_path, 'w') as fw:
-                for line in lines:
-                    fw.write(line.replace('_.xpi', 'ublock_latest.xpi'))
-
+        out = os.popen(
+            f'curl -sLo {Config.PROJECT_PATH}/ublock_latest.xpi {latest}'
+        ).read()
+        console.print(
+            '\nYou\'re running the "--mirroredto" flag '
+            'for the first time. Please wait until everything is ready. '
+            'This is a one-time thing.\n',
+            style='#f1fa8c')
+        time.sleep(5)
 
     def mirroredto(file: str):
-        if not shutil.which('firefox'):
-            console.print('[[#ff5555]CRITICAL![/#ff5555]] Cannot find Firefox!')
-            console.print(
-                'Install Firefox first: https://www.mozilla.org/en-US/firefox/all')
-            sys.exit(1)
+        def start_driver():
+            options = Options()
+            options.headless = True
+            try:
+                driver = webdriver.Firefox(options=options,
+                                           service_log_path=os.path.devnull)
+                driver.install_addon(Config.UBLOCK, temporary=True)
+                return driver
+            except WebDriverException:
+                os.environ['WDM_LOG_LEVEL'] = '0'
+                os.environ['WDM_PRINT_FIRST_LINE'] = 'False'
+                os.environ['WDM_LOCAL'] = '1'
+                driver = webdriver.Firefox(
+                    executable_path=GeckoDriverManager().install(),
+                    options=options,
+                    service_log_path=os.path.devnull)
+                driver.install_addon(Config.UBLOCK, temporary=True)
+                return driver
+            except Exception as e:
+                SeleniumExceptionInfo(e)
+                sys.exit(1)
 
-        if Path(Config.UBLOCK).name == '_.xpi':
+        check_ublock = [
+            x for x in glob(f'{Config.PROJECT_PATH}/*')
+            if Path(x).name == 'ublock_latest.xpi'
+        ]
+        if not check_ublock:
             PyMirror.download_ublock()
-            time.sleep(5)
 
         first_batch = [
             'GoFileIo', 'TusFiles', 'OneFichier', 'ZippyShare', 'UsersDrive',
@@ -272,29 +261,14 @@ class PyMirror:
         second_batch = [
             'GoFileIo', 'DownloadGG', 'TurboBit', 'Uptobox', 'SolidFiles',
             'DailyUploads', 'UploadEe', 'DropApk', 'MixdropCo', 'FilesIm',
-            'MegaupNet', 'dlupload', 'file-upload', 'catboxmoe'
+            'MegaupNet', 'dlupload', 'file-upload'
         ]
 
-        URLs = []
-
-        options = Options()
-        options.headless = True
-
-        try:
-            if platform.system() == 'Windows':
-                driver = webdriver.Firefox(executable_path=Config.WIN_GECKO,
-                                           options=options,
-                                           service_log_path=os.path.devnull)
-            else:
-                driver = webdriver.Firefox(options=options,
-                                           service_log_path=os.path.devnull)
-        except Exception:
-            driver = webdriver.Firefox(executable_path=GeckoDriverManager().install())
-            
-        driver.install_addon(Config.UBLOCK, temporary=True)
+        driver = start_driver()
 
         for ls in [first_batch, second_batch]:
             try:
+                time.sleep(2)
                 driver.get('https://www.mirrored.to/')
                 time.sleep(5)
                 html = driver.find_element_by_tag_name('html')
@@ -305,12 +279,14 @@ class PyMirror:
                         driver.find_element_by_id(x.lower()).click()
                     except Exception as e:
                         PyMirror.SeleniumExceptionInfo(e)
+                        continue
 
                 time.sleep(2)
+                send_file = lambda selector: driver.find_element_by_css_selector(
+                    selector).send_keys(file)
                 driver.find_element_by_css_selector(
                     '#uploadifive-html_file_upload > input[type=file]:nth-child(3)'
                 ).send_keys(file)
-
                 time.sleep(1)
 
                 driver.find_element_by_id('upload_button').click()
@@ -332,8 +308,8 @@ class PyMirror:
                 while True:
                     time.sleep(5)
                     status = [
-                        x.text
-                        for x in driver.find_elements_by_class_name('id_Success')
+                        x.text for x in driver.find_elements_by_class_name(
+                            'id_Success')
                     ]
                     if len(status) >= 8 or time.time() - start > 60:
                         break
@@ -348,10 +324,11 @@ class PyMirror:
                 for handle in driver.window_handles:
                     driver.switch_to.window(handle)
                     try:
-                        LINK = driver.find_element_by_class_name('code_wrap').text
-                        URLs.append(LINK)
-                        console.print(f'[[#50fa7b]OK[/#50fa7b]] {LINK}')
-                        logging.info(f'[OK] {LINK}')
+                        LINK = driver.find_element_by_class_name(
+                            'code_wrap').text
+                        all_links.append(LINK)
+                        console.print(f'[[#50fa7b] OK [/#50fa7b]] {LINK}')
+                        logger.info(f'[ OK ] {LINK}')
                         if handle != current_window:
                             driver.close()
                     except:
@@ -366,18 +343,16 @@ class PyMirror:
                 break
 
         driver.quit()
-        return URLs
 
-
-    def style_output(args, links: list, data: dict):
+    def style_output(args, LINKs: dict):
+        names = list(LINKs.keys())
+        links = list(LINKs.values())
         style = args.style
         if style == 'list':
             output = '\n'.join(links)
         elif style == 'markdown':
-            output = '\n'.join([
-                f'- [{name}]({link})'
-                for name, link in zip([k for k, _ in data.items()], links)
-            ])
+            output = '\n'.join(
+                [f'- [{name}]({link})' for name, link in zip(names, links)])
         elif style == 'reddit':
             output = ' | '.join(
                 [f'[Mirror {n + 1}]({link})' for n, link in enumerate(links)])
@@ -385,15 +360,17 @@ class PyMirror:
             output = '\n'.join(links)
         return output
 
-
     def uploader(self, args):
-        signal.signal(signal.SIGINT, KeyboardInterruptHandler.keyboardInterruptHandler)
+        signal.signal(signal.SIGINT,
+                      KeyboardInterruptHandler.keyboardInterruptHandler)
 
         with open(Config.DATA) as j:
             data = json.load(j)
 
         if args.log:
-            PyMirror.custom_logger(Config.LOG_FILE)
+            logger.remove()
+            logger.add(Config.LOG_FILE, level='DEBUG')
+            logger.add(sys.stderr, level='ERROR')
 
         if args.mirroredto is True:
             data['mirroredto'] = {'server': 'https://mirrored.to'}
@@ -421,17 +398,26 @@ class PyMirror:
             responses = [True] * len(data.keys())
 
         links_raw = PyMirror.api_uploads(args, data, responses, rfile)
-        links = PyMirror.match_links(links_raw)
+        # links = PyMirror.match_links(links_raw)
 
         if args.mirroredto is True:
             try:
                 urls = PyMirror.mirroredto(str(Path(rfile).resolve()))
             except WebDriverException as e:
                 PyMirror.SeleniumExceptionInfo(e)
-                urls = []
-            links = links + urls
 
-        output = PyMirror.style_output(args, links, data)
+        LINKs = {}
+
+        for link in all_links:
+            domain = link.split('/')[:-1][2].replace('www.', '')
+            name = '.'.join(domain.split('.')[0:])
+            if len(name) == 1:
+                name = '.'.join(domain.split('.')[1:])
+            elif len(name.split('.')) == 3:
+                name = '.'.join(domain.split('.')[1:])
+            LINKs.update({name: link})
+
+        output = PyMirror.style_output(args, LINKs)
 
         if args.delete:
             if Path(args.input).is_dir():
@@ -442,63 +428,7 @@ class PyMirror:
             os.remove(file)
         console.rule('Results')
         print(output)
-        for x in links:
-            logging.info(x)
+        os.rename(rfile, file)
+        for x in all_links:
+            logger.info(x)
         console.rule('END')
-
-
-# def main():
-
-#     parser = argparse.ArgumentParser(prog ='pymirror', formatter_class=fmt, add_help=False)
-#     parser.add_argument('-h',
-#                         '--help',
-#                         action='help',
-#                         default=argparse.SUPPRESS,
-#                         help='Show this help message and exit')
-#     parser.add_argument('-i',
-#                         '--input',
-#                         help='Path to the input file/folder')
-#     parser.add_argument('-s',
-#                         '--style',
-#                         help='Output style (default: lines)',
-#                         choices=['lines', 'list', 'markdown', 'reddit'],
-#                         default='lines')
-#     parser.add_argument(
-#         '-m',
-#         '--mirroredto',
-#         help='Use mirrored.to to generate more likes (default: False)',
-#         action='store_true',
-#         default=False)
-#     parser.add_argument(
-#         '-n',
-#         '--number',
-#         help='Select a specific number of servers to use (default: max)',
-#         type=int,
-#         default=None)
-#     parser.add_argument(
-#         '-d',
-#         '--delete',
-#         help='Delete the file after the process is complete (default: False)',
-#         action='store_true',
-#         default=False)
-#     parser.add_argument(
-#         '-c',
-#         '--check-status',
-#         help='Check the status of the remote servers (default: False)',
-#         action='store_false',
-#         default=False)
-#     parser.add_argument(
-#         '-l',
-#         '--log',
-#         help='Log the current uploadto a file (default: False)',
-#         action='store_true',
-#         default=False)
-#     parser.add_argument('-v',
-#                         '--verbose',
-#                         help='Make the process more talkative',
-#                         action='store_true',
-#                         default=False)
-
-#     args = parser.parse_args()
-
-#     PyMirror.pymirror(args)
